<?php

namespace Drupal\drd_pi;

use Drupal\drd\Entity\Domain;
use GuzzleHttp\Client;

/**
 * Provides platform based domain.
 */
class DrdPiDomain extends DrdPiEntity {

  protected $domain;
  protected $values;

  /** @var DrdPiCore */
  protected $core;

  /** @var \Drupal\drd\Logging */
  protected $logging;

  public function __construct($label, $type, $account, $id, $core, $domain) {
    parent::__construct($label, $type, $account, $id);
    $this->core = $core;
    $this->domain = $domain;
    $this->logging = \Drupal::service('drd.logging');
  }

  protected function ensureAgent() {
    // Overwrite this in an implementation of the specific PI implementation
    // and make sure the DRD Agent is available and installed.
    return TRUE;
  }

  protected function determineUrl($secure = TRUE) {
    $scheme = $secure ? 'https' : 'http';
    $url = $scheme . '://' . $this->domain;
    $success = FALSE;
    try {
      $client = new Client(['base_uri' => $url]);
      $response = $client->request('head');
      $statusCode = $response->getStatusCode();
      $success = ($statusCode < 300);
    }
    catch (\Exception $ex) {
      // Ignore
    }
    if (!$success) {
      \Drupal::service('drd.logging')->debug('Can not connect to @url', ['@url' => $url]);
      if ($secure) {
        return $this->determineUrl(FALSE);
      }
      return FALSE;
    }
    return $url;
  }

  protected function preSave() {
    /** @var \Drupal\drd\Entity\DomainInterface $domain */
    $domain = $this->getDrdEntity();
    $domain->initValues('');
  }

  protected function postSave() {
    /** @var \Drupal\drd\Entity\DomainInterface $domain */
    $domain = $this->getDrdEntity();
    /** @var \Drupal\drd\Entity\CoreInterface $core */
    $core = $this->core->getDrdEntity();
    if (empty($core->get('drupalroot')->value)) {
      $domain->initCore($core);
    }
    else {
      $domain->remoteInfo();
    }
  }

  /**
   * @inheritDoc
   */
  public function host() {
    return $this->core->host();
  }

  /**
   * @inheritDoc
   */
  public function create() {
    if (!$this->ensureAgent()) {
      $this->logging->log('emergency', 'Unable to ensure DRD Agent remotely');
      return $this;
    }
    if (!$url = $this->determineUrl()) {
      $this->logging->log('emergency', 'Unable to determine remote URL');
      return $this;
    }
    $this->entity = Domain::instanceFromUrl($this->core->getDrdEntity(), $url, [
      'pi_type' => $this->type,
      'pi_account' => $this->account,
      'pi_id_host' => $this->host()->id(),
      'pi_id_core' => $this->core->id(),
      'pi_id_domain' => $this->id,
    ]);
    $this->preSave();
    $this->entity->save();
    $this->postSave();
    return $this;
  }

}
