<?php

namespace Drupal\drd_pi_pantheon\Entity\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class AccountForm.
 */
class AccountForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    /** @var \Drupal\drd_pi_pantheon\Entity\AccountInterface $pantheon_account */
    $pantheon_account = $this->entity;
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $pantheon_account->label(),
      '#description' => $this->t('Label for the Account.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $pantheon_account->id(),
      '#machine_name' => [
        'exists' => '\Drupal\drd_pi_pantheon\Entity\Account::load',
      ],
      '#disabled' => !$pantheon_account->isNew(),
    ];

    $form['machine_token'] = [
      '#type' => 'textfield',
      '#default_value' => $pantheon_account->getMachineToken(),
      '#description' => $this->t('Obtain the machine token from your Pantheon dashboard in the "Account" section.'),
      '#required' => TRUE,
    ];

    /* You will need additional form elements for your custom properties. */

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $pantheon_account = $this->entity;
    $status = $pantheon_account->save();

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label Pantheon account.', [
          '%label' => $pantheon_account->label(),
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label Pantheon account.', [
          '%label' => $pantheon_account->label(),
        ]));
    }
    $form_state->setRedirectUrl($pantheon_account->toUrl('collection'));
  }

}
